//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define MaxSchedules 10

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FullIncrement(int);
void DateIncrement(int);
void TimeIncrement(int);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

const int MDays[12] =
{
     31, // 01 - Jan - 00 //
     28, // 02 - Feb - 01 //
     31, // 03 - Mar - 02 //
     30, // 04 - Apr - 03 //
     31, // 05 - May - 04 //
     30, // 06 - Jun - 05 //
     31, // 07 - Jul - 06 //
     31, // 08 - Aug - 07 //
     30, // 09 - Sep - 08 //
     31, // 10 - Oct - 09 //
     30, // 11 - Nov - 10 //
     31  // 12 - Dec - 11 //
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct _BackupTime{
    char *Name;
    char *Filename;
    char *Description;

    char *SQLServer;
    char *Database;
    char *DBDriver;
    char *Username;
    char *Password;
    BOOL IsTrusted;

    DWORD BackupTime;
    DWORD BackupDate;

    DWORD ScheduleID;

    WORD IncrementDay;
    WORD IncrementHour;
    WORD IncrementMinute;

    BOOL Active;
    BOOL ScheduleSet;
}; _BackupTime *BT;


struct _BackupOptionsDailog{
    char *BackupName;
    char *BackupDescription;
    char *BackupFileName;
    BOOL Complete;
}; _BackupOptionsDailog BOD;

struct _SQLOptionsDailog{
    char *SQLServer;
    char *DataBase;
    char *DBDriver;
    char *UserName;
    char *Password;
    BOOL IsTrusted;

    BOOL Complete;
}; _SQLOptionsDailog SQLOD;

struct _ScheduleDialog{
    HWND ListView_hWnd;
    int EditItem;
    LV_COLUMN LVColumb;
    LV_ITEM  LVItem;
}; _ScheduleDialog SD;

struct _EditScheduleDialog{
    int EditItem;
}; _EditScheduleDialog ESD;

struct _JobImformationDialog{
    HWND Database_hWnd;
    HWND Dialog_hWnd;
}; _JobImformationDialog JID;

struct _SQLPasswordDialog{
    HWND Dialog_hWnd;
    BOOL Cancel;
}; _SQLPasswordDialog SPWD;

int NumberOfSetSchedules = 0;
DWORD CurrentScheduleID  = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FixSchedule(int xSched)
{
    int IncrementMinute = BT[xSched].IncrementMinute;
    int IncrementHour   = BT[xSched].IncrementHour;
    int IncrementDay    = BT[xSched].IncrementDay;

    DWORD CurrentDate = 0;
    DWORD CurrentTime = 0;

    SYSTEMTIME ST;

    GetLocalTime(&ST);

    CurrentDate = DateToLong(ST.wYear, ST.wMonth,  ST.wDay);
    CurrentTime = TimeToLong(ST.wHour, ST.wMinute, ST.wSecond);

    if(IncrementDay == 0 && BT[xSched].BackupDate < CurrentDate)
        BT[xSched].BackupDate = CurrentDate;

    while(BT[xSched].BackupDate < CurrentDate)
    {
        DateIncrement(xSched);
        Sleep(1);
    }

    if(BT[xSched].BackupDate <= CurrentDate)
    {
        if(IncrementMinute == 0 && IncrementHour == 0)
        {
            BT[xSched].BackupTime = CurrentTime;
        }
        else{
            while(BT[xSched].BackupTime < CurrentTime)
            {
                TimeIncrement(xSched);
                Sleep(1);
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetSchedules()
{
    int LocalLoop = 0;

	while(LocalLoop < MaxSchedules)
	{
        if(BT[LocalLoop].ScheduleSet == TRUE)
        {
            FixSchedule(LocalLoop);
        }

        LocalLoop++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void LoadScheduleIntoDailog(void)
{
    int LocalLoop = 0;

	while(LocalLoop < MaxSchedules)
	{
        if(BT[LocalLoop].ScheduleSet == TRUE)
        {
    		char TempText[1024];

			SYSTEMTIME Date;
			SYSTEMTIME Time;

			GetSystemTime(&Date);
			GetSystemTime(&Time);

    		int Year, Month,  Day;
    		LongToDate(BT[LocalLoop].BackupDate, &Year, &Month,  &Day);
    		Date.wYear   = Year;
    		Date.wMonth  = Month;
    		Date.wDay    = Day;

    		int Hour, Minute, Second;
    		LongToTime(BT[LocalLoop].BackupTime, &Hour, &Minute, &Second);
    		Time.wHour   = Hour;
    		Time.wMinute = Minute;
    		Time.wSecond = Second;

            sprintf(TempText, "%d", BT[LocalLoop].ScheduleID);
            SD.LVItem.pszText  = TempText;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 0;
            ListView_InsertItem(SD.ListView_hWnd,&SD.LVItem);

            SD.LVItem.pszText  = BT[LocalLoop].Name;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 1;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);

            SD.LVItem.pszText  = BT[LocalLoop].Description;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 2;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);

            GetDateFormat(NULL, NULL, &Date, "MM-dd-yyyy", TempText, 1024);
            SD.LVItem.pszText  = TempText;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 3;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);

            GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &Time, NULL, TempText, 1024);
            SD.LVItem.pszText  = TempText;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 4;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);

            SD.LVItem.pszText  = BT[LocalLoop].Database;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 5;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);

        	SD.LVItem.pszText  = BT[LocalLoop].Filename;
            SD.LVItem.mask     = LVIF_TEXT;
            SD.LVItem.iSubItem = 6;
            ListView_SetItem(SD.ListView_hWnd,&SD.LVItem);
        }

        LocalLoop++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int ScheduleIDToPosition(DWORD ItemID)
{
    int LocalLoop = 0;
	while(LocalLoop < MaxSchedules)
	{
        if(BT[LocalLoop].ScheduleID == ItemID)
            return LocalLoop;

        LocalLoop++;
    }
    return -1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void EndScheduleByID(DWORD ItemID)
{
    int LocalLoop = 0;
	while(LocalLoop < MaxSchedules)
	{
        if(BT[LocalLoop].ScheduleID == ItemID)
        {
            BT[LocalLoop].ScheduleSet = FALSE;
            break;
        }
        LocalLoop++;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void InitSchedule(void)
{
    int LocalLoop = 0;

    BT = (_BackupTime *) calloc( sizeof(_BackupTime), MaxSchedules);

    while(LocalLoop < MaxSchedules)
    {
        BT[LocalLoop].Name             = NULL;
        BT[LocalLoop].Filename         = NULL;
        BT[LocalLoop].Description      = NULL;

        BT[LocalLoop].SQLServer        = NULL;
        BT[LocalLoop].Database         = NULL;
        BT[LocalLoop].DBDriver         = NULL;
        BT[LocalLoop].Username         = NULL;
        BT[LocalLoop].Password         = NULL;
        BT[LocalLoop].IsTrusted        = FALSE;

        BT[LocalLoop].BackupTime       = 0;
        BT[LocalLoop].BackupDate       = 0;
        BT[LocalLoop].IncrementDay     = 0;
        BT[LocalLoop].IncrementHour    = 0;
        BT[LocalLoop].IncrementMinute  = 0;
        BT[LocalLoop].ScheduleID       = 0;
        BT[LocalLoop].Active           = FALSE;
        BT[LocalLoop].ScheduleSet      = FALSE;

        LocalLoop++;
    }

    NumberOfSetSchedules = 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NextAvailSchedule(void)
{
    int LocalLoop = 0;

    while(LocalLoop < MaxSchedules)
    {
        if(BT[LocalLoop].ScheduleSet == FALSE)
            return LocalLoop;

        LocalLoop++;
    }

    return -1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CheckSchedule(void)
{
    SYSTEMTIME ST;

    int LocalLoop    = 0;
    DWORD CurrentDate = 0;
    DWORD CurrentTime = 0;

    if(CurrentlyRunning == TRUE)
        return -1;

    GetLocalTime(&ST);

    CurrentDate = DateToLong(ST.wYear, ST.wMonth,  ST.wDay);
    CurrentTime = TimeToLong(ST.wHour, ST.wMinute, ST.wSecond);

    while(LocalLoop < MaxSchedules)
    {
        if(BT[LocalLoop].ScheduleSet == TRUE && BT[LocalLoop].Active == TRUE)
        {
            if(BT[LocalLoop].BackupDate == CurrentDate)
            {
                if(CurrentTime >= BT[LocalLoop].BackupTime)
                {
                    FullIncrement(LocalLoop);
                    return LocalLoop;
                }
            }
        }

        LocalLoop++;
    }

    return -1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FullIncrement(int Sched)
{
    int LocalLoop = 0;
    int Year      = 0;
    int Month     = 0;
    int Day       = 0;
    int Hour      = 0;
    int Minute    = 0;
    int Second    = 0;

    int IncrementMinute = BT[Sched].IncrementMinute;
    int IncrementHour   = BT[Sched].IncrementHour;
    int IncrementDay    = BT[Sched].IncrementDay;
    int IncrementMonth  = 0;

    LongToDate(BT[Sched].BackupDate, &Year, &Month,  &Day);
    LongToTime(BT[Sched].BackupTime, &Hour, &Minute, &Second);

    //---------------------------------------------------

    //---------( Increment Minute )
    LocalLoop = 0;
    while(LocalLoop != IncrementMinute)
    {
        Minute++;
        LocalLoop++;

        if(Minute >= 60)
        {
            Minute = 0;
            IncrementHour++;
        }
    }
    //---------------------------------------------------

    //---------( Increment Hour )
    LocalLoop = 0;
    while(LocalLoop != IncrementHour)
    {
        Hour++;
        LocalLoop++;

        if(Hour >= 24)
        {
            Hour = 0;
            IncrementDay++;
        }
    }
    //---------------------------------------------------

    //---------( Increment Day )
    LocalLoop = 0;
    while(LocalLoop != IncrementDay)
    {
        Day++;
        LocalLoop++;

        if(Day > MDays[ (Month + IncrementMonth) - 1 ])
        {
            Day = 1;
            IncrementMonth++;
        }
    }
    //---------------------------------------------------

    //---------( Increment Month )
    LocalLoop = 0;
    while(LocalLoop != IncrementMonth)
    {
        Month++;
        LocalLoop++;

        if(Month == 13)
        {
            Month = 1;
            Year++;
        }
    }

    //---------------------------------------------------

    BT[Sched].BackupDate = DateToLong(Year, Month,  Day);
    BT[Sched].BackupTime = TimeToLong(Hour, Minute, Second);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TimeIncrement(int Sched)
{
    int LocalLoop = 0;
    int Hour      = 0;
    int Minute    = 0;
    int Second    = 0;

    int IncrementMinute = BT[Sched].IncrementMinute;
    int IncrementHour   = BT[Sched].IncrementHour;

    LongToTime(BT[Sched].BackupTime, &Hour, &Minute, &Second);

    //---------------------------------------------------

    //---------( Increment Minute )
    LocalLoop = 0;
    while(LocalLoop != IncrementMinute)
    {
        Minute++;
        LocalLoop++;

        if(Minute >= 60)
        {
            Minute = 0;
            IncrementHour++;
        }
    }
    //---------------------------------------------------

    //---------( Increment Hour )
    LocalLoop = 0;
    while(LocalLoop != IncrementHour)
    {
        Hour++;
        LocalLoop++;

        if(Hour >= 24)
        {
            Hour = 0;
        }
    }
    //---------------------------------------------------

    BT[Sched].BackupTime = TimeToLong(Hour, Minute, Second);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void DateIncrement(int Sched)
{
    int LocalLoop = 0;
    int Year      = 0;
    int Month     = 0;
    int Day       = 0;

    int IncrementDay    = BT[Sched].IncrementDay;
    int IncrementMonth  = 0;

    LongToDate(BT[Sched].BackupDate, &Year, &Month,  &Day);

    //---------------------------------------------------

    //---------( Increment Day )
    LocalLoop = 0;
    while(LocalLoop != IncrementDay)
    {
        Day++;
        LocalLoop++;

        if(Day > MDays[ (Month + IncrementMonth) - 1 ])
        {
            Day = 1;
            IncrementMonth++;
        }
    }
    //---------------------------------------------------

    //---------( Increment Month )
    LocalLoop = 0;
    while(LocalLoop != IncrementMonth)
    {
        Month++;
        LocalLoop++;

        if(Month == 13)
        {
            Month = 1;
            Year++;
        }
    }

    //---------------------------------------------------

    BT[Sched].BackupDate = DateToLong(Year, Month,  Day);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

